import re
import json


class Method:
    def modify_keyboard(self, buttons, inline=False):
        new_buttons = []
        for row in buttons:
            new_row = []
            for button in row:
                if isinstance(button, dict):
                    if inline and "callback_data" in button:
                        new_row.append(button)
                    elif not inline and "callback_data" not in button:
                        new_row.append(button)
                elif not inline:
                    new_row.append(button)
            if new_row:
                new_buttons.append(new_row)
        return new_buttons

    def create_keyboard(self, buttons):
        return json.dumps(
            {
                "inline_keyboard": self.modify_keyboard(buttons, inline=True),
                "keyboard": self.modify_keyboard(buttons, inline=False),
                "one_time_keyboard": True,
            }
        )

    def __getattr__(self, name):
        async def wrapper(**params):
            camel_case_name = re.sub(r"_([a-z])", lambda m: m.group(1).upper(), name)
            return await self.request(camel_case_name, params)

        return wrapper



from telegant.method import Method
import aiohttp
import asyncio


class EventHandler(Method):
    def __init__(self):
        self.chat_id = 0
        self.handlers = {}
        self.message_handlers = {}
        self.command_handlers = {}
        self.callback_handlers = {}
        self.update_handlers = {}

    def add_handler(self, handler_dict, key):
        def decorator(handler):
            handler_dict[key] = handler
            return handler

        return decorator

    async def request(self, action, params=None):
        async with aiohttp.ClientSession() as session:
            try:
                url = f"{self.base_url}{action}"
                if not params.get("chat_id"):
                    params["chat_id"] = self.chat_id
                if params.get("reply_markup"):
                    params["reply_markup"] = self.create_keyboard(params["reply_markup"])
                response = await session.post(url, params=params)
                return await response.json()
            except Exception as e:
                print(f"Error sending request: {e}")

    async def handle_update(self, update):
        tasks = []
        for key in update:
            if (handler := self.handlers.get(key)) is not None:
                for task in handler:
                    if task is not None:
                        tasks.append(asyncio.create_task(task.handle(update)))
        await asyncio.gather(*tasks)



import re


class TextHandler:
    def __init__(self, event_handler):
        self.event_handler = event_handler

    async def handle(self, update):
        self.event_handler.chat_id = update["message"]["from"]["id"]
        message_text = update["message"]["text"]

        for pattern, handler in self.event_handler.message_handlers.items():
            if re.fullmatch(pattern, message_text):
                await handler(self.event_handler, update)
                return



from telegant.handlers import (
    EventHandler,
    TextHandler,
    CommandHandler,
    CallbackQueryHandler,
    UpdateHandler,
)

import re
import aiohttp
import asyncio
import json


class Bot:
    def __init__(self, token):
        self.token = token
        self.event_handler = EventHandler()
        self.event_handler.base_url = f"https://api.telegram.org/bot{self.token}/"

    async def polling(self):
        last_update_id = 0
        async with aiohttp.ClientSession() as session:
            while True:
                response_json, last_update_id = await self.get_updates(
                    session, last_update_id
                )
                if not response_json.get("ok"):
                    print("Error: Response is not OK")
                    continue

                for update in response_json["result"]:
                    await self.event_handler.handle_update(update)

    async def get_updates(self, session, last_update_id):
        try:
            response = await session.get(
                f"{self.event_handler.base_url}getUpdates",
                params={"offset": last_update_id, "timeout": 30},
            )
            if response.status != 200:
                print(f"Error: {response.status}")
                return None, last_update_id

            response_json = await response.json()
            for update in response_json["result"]:
                last_update_id = max(last_update_id, update["update_id"] + 1)

            return response_json, last_update_id

        except Exception as e:
            print(f"Error polling for updates: {e}")
            return None, last_update_id

    def start_polling(self):
        asyncio.run(self.polling())

    def process_event_handler(self, value, key, handler, handlers):
        handlers = getattr(self.event_handler, handlers)

        if key not in self.event_handler.handlers:
            self.event_handler.handlers[key] = []

        if type(handler) in (type(h) for h in self.event_handler.handlers[key]):
            return self.event_handler.add_handler(handlers, value)

        self.event_handler.handlers[key].extend([handler])
        return self.event_handler.add_handler(handlers, value)

    def process_many_events(
        self, events_list, event_type, handler_cls, handler_list_attr
    ):
        def decorator(handler_func):
            for event in events_list:
                self.process_event_handler(
                    event, event_type, handler_cls, handler_list_attr
                )(handler_func)
            return handler_func

        return decorator

    def with_args(self, keys):
        def decorator(handler_func):
            async def wrapper(bot, update, data):
                message = update.get("message")
                if message:
                    message_text = message.get("text", "")
                    args = message_text.split()[1:]
                    data = {
                        k: args[i] if i < len(args) else "" for i, k in enumerate(keys)
                    }
                    await handler_func(bot, update, data)

            return wrapper

        return decorator

    def hear(self, value):
        return self.process_event_handler(
            value, "message", TextHandler(self.event_handler), "message_handlers"
        )

    def hears(self, value):
        return self.process_many_events(
            value, "message", TextHandler(self.event_handler), "message_handlers"
        )

    def command(self, value):
        return self.process_event_handler(
            value, "message", CommandHandler(self.event_handler), "command_handlers"
        )

    def callback(self, value):
        return self.process_event_handler(
            value,
            "callback_query",
            CallbackQueryHandler(self.event_handler),
            "callback_handlers",
        )

    def commands(self, commands_list):
        return self.process_many_events(
            commands_list,
            "message",
            CommandHandler(self.event_handler),
            "command_handlers",
        )

    def callbacks(self, callbacks_list):
        return self.process_many_events(
            callbacks_list,
            "callback_query",
            CallbackQueryHandler(self.event_handler),
            "callback_handlers",
        )



class UpdateHandler:
    def __init__(self, event_handler):
        self.event_handler = event_handler

    async def handle(self, update):
        raise NotImplementedError("This function has not been implemented yet")



class CallbackQueryHandler:
    def __init__(self, event_handler):
        self.event_handler = event_handler

    async def handle(self, update):
        self.event_handler.chat_id = update["callback_query"]["from"]["id"]
        callback_data = update["callback_query"]["data"]
        message = update["callback_query"].get("message")

        handler = self.event_handler.callback_handlers.get(callback_data)
        if handler is not None:
            await handler(self.event_handler, update, message)

        await self.answer_callback_query(update["callback_query"]["id"])

    async def answer_callback_query(self, callback_query_id):
        method = "answerCallbackQuery"
        params = {"callback_query_id": callback_query_id}
        await self.event_handler.request(method, params)



class CommandHandler:
    def __init__(self, event_handler):
        self.event_handler = event_handler

    async def handle(self, update):
        self.event_handler.chat_id = update["message"]["from"]["id"]
        message_text = update["message"]["text"]

        command, *args = message_text[1:].split()
        handler = self.event_handler.command_handlers.get(command)
        if handler:
            await handler(self.event_handler, update, args)
            return



from .EventHandler import EventHandler
from .CommandHandler import CommandHandler
from .TextHandler import TextHandler
from .CallbackQueryHandler import CallbackQueryHandler
from .UpdateHandler import UpdateHandler
